local name, addon_table = ...

local RW = LibStub("AceAddon-3.0"):NewAddon("Raid Watch 3", "AceEvent-3.0")
RW3 = RW

local Debug = addon_table.Debug
local expect = Debug.expect
RW.Debug = Debug

RW.DEBUG = true
local DEBUG = RW.DEBUG

local Callbacks = LibStub("CallbackHandler-1.0"):New(RW)
RW.Callbacks = Callbacks

local ConfigMode = false

RW.Load = {
	Enabled = {},
	CoreEnabled = {},
	CoreLoaded = {},
	CoreInitialized = {},
}

local Events = {}

function RW:OnInitialize()
	local defaults = {
		profile = {
			DebugLevel = 0,
			Color = {1, 0, 1, 1},
		}
	}
	
	self.db = LibStub("AceDB-3.0"):New("RaidWatch3DB", defaults, true)
	defaults = nil
	
	Debug.db = self.db
	
	self:InitModLists()
	
	self.Utils:NamedTable(Events, "Loader_Events")
	self.RegisterCallback(Events, "Enabled")
	self.RegisterCallback(Events, "CoreLoaded")
	self.RegisterCallback(Events, "CoreInitialized")
	self.RegisterCallback(Events, "CoreEnabled")
	
	local LDB = self.LDB:RegisterLDB("Raid Watch 3", "data source")
	self.LDBObj = LDB
end

function RW:OnEnable()
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	
	Callbacks:Fire("Enabled")
	
	self.LDBObj:SetIcon(self.Utils:IconTexture("INV_Misc_EngGizmos_01"))
	self.LDBObj:AddClickHelpLine("LoadAndShowOptions", "Left Click to Load and Show options", "Left")
	self.LDBObj:AddClickHelpLine("ToggleConfigMode", "Right Click to toggle config mode.", "Right")
	self.LDBObj:AddClickFunction("Left", "LoadConfig", function()
		RW:LoadAddOn({ ID = "RaidWatch3_Config", Name = "RaidWatch 3 [Config]" })
		RW.LDBObj:RemoveClickFunction("Left", "LoadConfig")
		RW.LDBObj:RemoveClickHelpLine("LoadAndShowOptions")
		if RW.Config then
			RW.Config:Toggle()
		end
	end)
	self.LDBObj:AddClickFunction("Right", "ToggleConfigMode", function()
		ConfigMode = not ConfigMode
		if ConfigMode then
			RW.LDBObj:SetText("Config Mode")
		else
			RW.LDBObj:SetText("Raid Watch 3")
		end
		Callbacks:Fire("ToggleConfigMode", ConfigMode)
	end)
	self.LDBObj:SetText("Raid Watch 3")	
end

function RW:ZONE_CHANGED_NEW_AREA(event)
	
end

function RW:PLAYER_ENTERING_WORLD(event)
	self:ZONE_CHANGED_NEW_AREA("ZONE_CHANGED_NEW_AREA")
end

function Events:Enabled()
	for _, AddOn in pairs(RW.Load.Enabled) do
		RW:LoadAddOn(AddOn)
	end
end

function Events:CoreLoaded()
	RW.UnregisterCallback(self, "CoreLoaded")
end

function Events:CoreInitialized()
	RW.UnregisterCallback(self, "CoreInitialized")
end

function Events:CoreEnabled()
	RW.UnregisterCallback(self, "CoreEnabled")
	for _, AddOn in pairs(RW.Load.CoreEnabled) do
		RW:LoadAddOn(AddOn)
	end
end