local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch 3")

local function CallPluginFunction(plugin, functionName, ...)
	if plugin[functionName] and type(plugin[functionName]) == "function" then
		plugin[functionName](plugin, ...)
	end
end

local Plugin = {}

function RW:NewPlugin(name, ...)
	local plugin = RW:NewModule(name, Plugin, ...)
	
	plugin.Constants = setmetatable({}, {
		__index = function(self, k)
			if type(k) == "number" then
				for i, v in pairs(self) do
					if v == k then
						return i
					end
				end
			elseif rawget(self, k) then
				return rawget(self, k)
			end
		end,
		__newindex = function(self, k, value)
			for i, v in pairs(self) do
				if v == value then
					error(("Cannot register constant %q with value %d, %q allready exsists"):format(k, value, i))
				end
			end
			rawset(self, k, value)
		end,
	})

	--- Register a new constant
	-- @name Plugin.Constants:Register
	-- @param name Name of the new constant
	-- @param value Value of the new constant
	function plugin.Constants:Register(name, value)
		if DEBUG then
			expect(name, "typeof", "string")
			expect(value, "typeof", "number")
			expect(name, "~=", "Register")
		end
		
		if self[name] then
			error(("Cannot register constant %q, it's allready registered."):format(name))
		end
		
		self[name] = value
	end
	
	return plugin
end

function Plugin:OnInitialize()
	RW.RegisterCallback(self, "ConfigLoaded", "_ConfigLoaded")
	RW.RegisterCallback(self, "ToggleConfigMode", "_ToggleConfigMode")
	
	CallPluginFunction(self, "OnPluginInitialize")
	
	-- Register DB Callbacks
	if self.db and self.OnReset then
		self.db.RegisterCallback(self, "OnProfileChanged", "OnReset")
		self.db.RegisterCallback(self, "OnProfileCopied", "OnReset")
		self.db.RegisterCallback(self, "OnProfileReset", "OnReset")
	end
end

function Plugin:OnEnable()
	CallPluginFunction(self, "OnPluginEnable")
end

function Plugin:OnDisable()
	CallPluginFunction(self, "OnPluginDisable")
end

function Plugin:ToggleEnabled(enabled)
	if enabled and not self:IsEnabled() then
		self:Enable()
	elseif not enabled and self:IsEnabled() then
		self:Disable()
	end
end

function Plugin:_ConfigLoaded(...)
	RW.UnregisterCallback(self, "ConfigLoaded")
	CallPluginFunction(self, "OnConfigLoaded", ...)
end

function Plugin:_ToggleConfigMode(...)
	CallPluginFunction(self, "ToggleConfigMode", ...)
end